﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ProgUstredni;
using RaketaAuto;
using ConsoleBufferApi.Input;

namespace RaketaRucne
{
    public partial class Form1 : Form
    {
        Image lander_img;
        Raketa raketa;
        Pilot pilot;
        bool first = true, autopilot = false;
        float scale_g = 1;

        Font fontBig, font;
        Timer timer;

        public Form1()
        {
            InitializeComponent();

            this.Load += new EventHandler(Form1_Load);
            this.Resize += new EventHandler(Form1_Resize);
        }

        void Form1_Resize(object sender, EventArgs e)
        {
            if (timer.Enabled)
            {
                this.redraw();
            }
        }

        void Form1_Load(object sender, EventArgs e)
        {
            this.lander_img = Image.FromFile("lunar_lander_neg.png");
            this.fontBig = new Font("Courier New", 30f, FontStyle.Bold);
            this.font = new Font("Courier New", 12f);

            timer = new Timer();
            timer.Interval = 50;
            timer.Tick += new EventHandler(t_Tick);
            timer.Enabled = false;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            label2.Visible = false;
            this.Cursor = Cursors.WaitCursor;
            autopilot = false;
            
            if (textBox1.Text.Split(' ').Length > 1)
            {
                if (textBox1.Text.Split(' ')[0] == "autopilot")
                {
                    autopilot = true;
                    textBox1.Text = textBox1.Text.Split(' ')[1];
                }
            }

            try
            {
                Http.Get("http://" + textBox1.Text);

                raketa = new Raketa(textBox1.Text);
                if (autopilot)
                    pilot = new Pilot(raketa, -15);

                this.Cursor = Cursors.Default;
                textBox1.Visible = false;
                button1.Visible = false;
                label1.Visible = false;
                label2.Visible = false;
                timer.Start();
            }
            catch (Exception exc)
            {
                label2.Visible = true;
            }

            this.Cursor = Cursors.Default;
        }

        void t_Tick(object sender, EventArgs e)
        {
            if (first)
            {
                first = false;

                raketa.Start();
            }

            if (!autopilot)
            {
                raketa.Nahoru = Keyboard.IsKeyDown(ConsoleBufferApi.Input.Keys.Up);
                raketa.Doleva = Keyboard.IsKeyDown(ConsoleBufferApi.Input.Keys.Left);
                raketa.Doprava = Keyboard.IsKeyDown(ConsoleBufferApi.Input.Keys.Right);
            }
            else
            {
                pilot.Update();
            }

            raketa.Update();
            this.redraw();
        }

        // kresleni trysek
        void tryskej(PointF start, PointF smer, Graphics g)
        {
            PointF[] points = new PointF[20];
            Random rnd = new Random();

            float length = (float)Math.Sqrt(smer.X * smer.X + smer.Y * smer.Y);

            for (int i = 0; i < points.Length; i++)
            {
                if (i % 2 == 0)
                {
                    points[i] = new PointF(
                            start.X + .1f * length * (float)(rnd.NextDouble() - 0.5),
                            start.Y + .1f * length * (float)(rnd.NextDouble() - 0.5)
                        );
                }
                else
                {
                    points[i] = new PointF(
                            start.X + smer.X + .5f * length * (float)(rnd.NextDouble() - 0.5),
                            start.Y + smer.Y + .5f * length * (float)(rnd.NextDouble() - 0.5)
                        );
                }
            }

            g.DrawLines(Pens.Red, points);
        }

        // hlavni kreslici smycka
        void redraw()
        {
            Bitmap buffer = new Bitmap(this.ClientSize.Width, this.ClientSize.Height);

            int width = this.ClientSize.Width;
            int height = this.ClientSize.Height;

            int bottom = this.ClientSize.Height - 50;

            float scale = Math.Min(1.5f, 200f / raketa.Y);
            if (scale_g < 0) scale_g = scale;
            scale_g = scale_g * 0.9f + scale * 0.1f;

            float shiftX = -scale_g * (raketa.R + raketa.L) / 2 + (width / 2);


            Graphics g = Graphics.FromImage(buffer);
            g.Clear(Color.Black);


            // trysky
            if (!raketa.konec)
            {
                if (raketa.Nahoru)
                    tryskej(new PointF(scale_g * (raketa.X) + shiftX, bottom - scale_g * (raketa.Y - 10)),
                        new PointF(0, 15 * scale_g), g);
                if (raketa.Doprava)
                    tryskej(new PointF(scale_g * (raketa.X - 7) + shiftX, bottom - scale_g * (raketa.Y + 3)),
                        new PointF(-10 * scale_g, 0), g);
                if (raketa.Doleva)
                    tryskej(new PointF(scale_g * (raketa.X + 7) + shiftX, bottom - scale_g * (raketa.Y + 3)),
                        new PointF(10 * scale_g, 0), g);
            }

            // zem
            g.DrawLine(Pens.Gray, new Point(0, bottom), new Point(width, bottom));

            // prist. plocha
            g.DrawLine(new Pen(Brushes.LightGray, 2f),
                    new PointF(scale_g * raketa.L + shiftX, bottom),
                    new PointF(scale_g * raketa.R + shiftX, bottom)
                );

            // raketa
            g.DrawImage(lander_img, new RectangleF(
                    scale_g * (raketa.X - 10) + shiftX, // X
                    bottom - scale_g * (raketa.Y + 10), // Y
                    scale_g * 20, scale_g * 20)         // size
                );


            // konec?
            if (raketa.konec)
            {
                PointF text_pos =  new PointF(0.3f * width, 0.1f * height);

                if (raketa.zdar)
                {
                    g.DrawString("Zdar!", fontBig, Brushes.Green, text_pos);
                }
                else
                {
                    g.DrawString("Krach!", fontBig, Brushes.Red, text_pos);
                }
            }


            g.DrawString("X: " + raketa.X.ToString("F1"), font, Brushes.White, new PointF(10, bottom + 5));
            g.DrawString("Y: " + raketa.Y.ToString("F1"), font, Brushes.White, new PointF(10, bottom + 23));

            g.DrawString("Vx: " + raketa.Vx.ToString("F1"), font, Brushes.White, new PointF(100, bottom + 5));
            g.DrawString("Vy: " + raketa.Vy.ToString("F1"), font, Brushes.White, new PointF(100, bottom + 23));

            g.DrawString("Palivo: " + raketa.Palivo.ToString("F1"), font, Brushes.White, new PointF(220, bottom + 5));
            g.DrawString("[                         ]", font, Brushes.White, new PointF(210, bottom + 23));
            g.FillRectangle(Brushes.White, new RectangleF(220, bottom + 29, 253 * raketa.Palivo / 20f, 8f));

            this.CreateGraphics().DrawImageUnscaled(buffer, Point.Empty);
        }
    }
}
